@@ -2,8 +2,11 @@
 from __future__ import absolute_import, print_function
 
 import os
+import sys
 import json
 import base64
+import atexit
+import signal
 import traceback
 from suanpan.g import g
 from suanpan.log import logger
@@ -13,9 +16,25 @@ from suanpan.arguments import suanpanTypes
 
 
 class APP:
+
+    if sys.platform == "win32":
+        SIGNALS = (
+            signal.SIGTERM,
+            signal.SIGINT,
+        )
+    else:
+        SIGNALS = (
+            signal.SIGUSR1,
+            signal.SIGTERM,
+            signal.SIGQUIT,
+            signal.SIGHUP,
+            signal.SIGINT,
+        )
+
     def __init__(self):
         self.func = None
         self.afterInitFunc = None
+        self.beforeExitFunc = None
         nodeinfo = self.loadNodeInfo()
         self.inputArgs = self.addArguments(
             self.parseNodeInfo("inputs", nodeinfo))
@@ -44,10 +63,23 @@ class APP:
     def __call__(self, func):
         self.func = func if func else self.func
 
+    def _exit(self, signum, frame):
+        logger.debug(f"Signal Exit: {signum} {frame}")
+        sys.exit(signum)
+
     def afterInit(self, func):
         self.afterInitFunc = func if func else self.afterInitFunc
 
-    def start(self):
+    def beforeExit(self, func):
+        self.beforeExitFunc = func if func else self.beforeExitFunc
+
+    def registerBeforeExitHooks(self, *args, **kwargs):
+        atexit.register(self.beforeExitFunc, *args, **kwargs)
+        for s in self.SIGNALS:
+            signal.signal(s, self._exit)
+        return self
+
+    def start(self, *args, **kwargs):
         logger.info("Start Running...")
         try:
             self.paramContext = self.loadParams(g)
@@ -59,6 +91,14 @@ class APP:
                 f"Load parameters or Run afterinit function error: {tracebackInfo}"
             )
             os._exit(0)
+        if self.beforeExitFunc:
+            try:
+                self.registerBeforeExitHooks(Context(args=self.paramContext))
+            except Exception as e:
+                tracebackInfo = traceback.format_exc()
+                logger.error(
+                    f"Register beforeExit function error: {tracebackInfo}")
+                os._exit(0)
         for context in stream.subscribeMessage():
             try:
                 inputContext = self.loadInputs(context)
@@ -99,13 +139,13 @@ class APP:
                                                        "inputData")].load(v))
         return params
 
-    def saveOutputs(self, context):
+    def saveOutputs(self, result, context):
         logger.debug("Saving Output Arguments:")
         params = {}
-        for k, v in context.items():
+        for k, v in result.items():
             if k.startswith("out") and k[-1].isdigit():
-                result = self.outputArgs[k.replace("out",
-                                                   "outputData")].dump(v)
+                result = self.outputArgs[k.replace("out", "outputData")].dump(
+                    v, requestId=context["request_id"].replace("-", ""))
                 for port, value in list(result.items()):
                     if port.startswith("out"):
                         port = port.replace(
@@ -129,7 +169,7 @@ class APP:
 
     def send(self, result, context):
         result = self.formatOutput(result)
-        result = self.saveOutputs(result)
+        result = self.saveOutputs(result, context)
         logger.debug("Sending Result to Next Nodes...")
         stream.sendMessage(result, context)
 
